﻿/***************************************************************
 * Copyright: xiaotie@gmail.com(http://xiaotie.cnblogs.com)
 * *************************************************************/
using System;
using System.Collections.Generic;
using System.Reflection;
using System.Globalization;
using Microsoft.CSharp;
using System.CodeDom;
using System.CodeDom.Compiler;
using System.Text;
using System.IO;
using System.Xml;

namespace Orc.Shell.Core
{
    public class Context
    {
        public static String EnsureAtLeastLength(String input, Int32 length)
        {
            String output = input;
            if (input.Length < length)
            {
                int count = length - input.Length;
                for (int i = 0; i < count; i++)
                {
                    output += " ";
                }
            }

            return output;
        }

        public Boolean Debug { get; set; }

        public IDictionary<String, Assembly> Assemblys { get; set; }
        public TypeManager TypeManager { get; set; }
        public SortedDictionary<String, String> AliasCmds { get; set; }
        public SortedDictionary<String, Object> Instances {get;set;}

        public CSharpCodeProvider CodeProvider { get; set; }

        public CompilerParameters CompilerParameters { get; set; }

        public Object this[String instanceName]
        {
            get
            {
                if(Instances.ContainsKey(instanceName))
                {
                    return Instances[instanceName];
                }
                else
                {
                    return null;
                }
            }
            set
            {
                if(Instances.ContainsKey(instanceName))
                {
                    Instances.Remove(instanceName);
                }
                Instances.Add(instanceName,value);
            }
        }

        public Context()
        {
            Debug = false;
            AliasCmds = new SortedDictionary<string, string>();
            LoadAliasCmds();

            CodeProvider = new CSharpCodeProvider(new Dictionary<string, string>() { { "CompilerVersion", "v3.5" } });
            CompilerParameters = new CompilerParameters(new[] { "mscorlib.dll", "System.Core.dll", "Orc.Shell.Core.dll", "OrcShell.exe" });
            CompilerParameters.GenerateExecutable = false;
            CompilerParameters.GenerateInMemory = true;

            Instances = new SortedDictionary<String,Object>();
            TypeManager = new TypeManager();

            Assemblys = new Dictionary<String, Assembly>();

            Assembly[] al = AppDomain.CurrentDomain.GetAssemblies();
            foreach (Assembly a in al)
            {
                AddAssembly(a);
            }

            AppDomain.CurrentDomain.AssemblyLoad += new AssemblyLoadEventHandler(CurrentDomain_AssemblyLoad);
        }

        private void LoadAliasCmds()
        {
            String path = AppDomain.CurrentDomain.BaseDirectory + "Alias.xml";
            if (File.Exists(path))
            {
                XmlDocument doc = new XmlDocument();
                doc.Load(path);
                XmlNodeList xnl = doc["data"].ChildNodes;
                foreach (XmlNode xn in xnl)
                {
                    String sc = xn.Attributes["shortCmd"].Value;
                    String fc = xn.Attributes["fullCmd"].Value;
                    if (!AliasCmds.ContainsKey(sc))
                    {
                        AliasCmds.Add(sc, fc);
                    }
                }
            }
        }

        private void AddAssembly(Assembly a)
        {
            if (a != null)
            {
                Assemblys.Add(a.FullName, a);
                Type[] tl = a.GetTypes();

                foreach (Type t in tl)
                {
                    if(!t.FullName.StartsWith("<PrivateImplementationDetails>"))
                        TypeManager.AddType(t);
                }
            }
        }

        void CurrentDomain_AssemblyLoad(object sender, AssemblyLoadEventArgs args)
        {
            Assembly a = args.LoadedAssembly;
            if (!Assemblys.ContainsKey(a.FullName))
            {
                AddAssembly(a);
            }
        }

        public void Invoke(String cmd)
        {
            try
            {
                CmdDispatcher pt = new CmdDispatcher(this, cmd);
                pt.Dispatch();
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
            }
        }

        public void Save(String argName, Object arg)
        {
            this[argName] = arg;
        }

        public void ListInstances()
        {
            foreach (String key in Instances.Keys)
            {
                Console.WriteLine(key + "\t" + Instances[key]);
            }
        }

        public void ListAlias()
        {
            foreach (String sc in AliasCmds.Keys)
            {
                Console.WriteLine(sc + "\t" + AliasCmds[sc]);
            }
        }

        public String GetFullCmd(String shortCmd)
        {
            if (AliasCmds.ContainsKey(shortCmd))
            {
                return AliasCmds[shortCmd];
            }
            return null;
        }
    }
}
